/*  Generic C List  Copyright (C) 2008  Giovanni Santostefano
    This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.

The hypothetical commands `show w' and `show c' should show the appropriate
parts of the General Public License.  Of course, your program's commands
might be different; for a GUI interface, you would use an "about box".

  You should also get your employer (if you work as a programmer) or school,
if any, to sign a "copyright disclaimer" for the program, if necessary.
For more information on this, and how to apply and follow the GNU GPL, see
<http://www.gnu.org/licenses/>.

  The GNU General Public License does not permit incorporating your program
into proprietary programs.  If your program is a subroutine library, you
may consider it more useful to permit linking proprietary applications with
the library.  If this is what you want to do, use the GNU Lesser General
Public License instead of this License.  But first, please read
<http://www.gnu.org/philosophy/why-not-lgpl.html>.
*/

#include "genlist.h"


/*Adds a node to the list
 * returns the new head of the list
 */
struct GenList* GList_addNode(void *item, /*the item void ptr*/
			      enum DEFTYPES tp, /*the type of the item*/
			      struct GenList *head /*the head of the list*/
			      )
{
  struct GenList *temp=NULL;

  //allocate the new node
  temp=malloc(sizeof(struct GenList));
  if(!temp)
    {
      fprintf(stderr,"Could not allocate memory");
      return NULL;
    }

  temp->next=head;
  temp->itemtype=tp;
  temp->itemptr=item;

  return temp;
}

/*Removes the head node
 *pass the head of the list
 *returns the next node
 */
struct GenList* GList_removeHeadNode(struct GenList *head)
{
  struct GenList *temp=head->next;
  free(head);
  return temp;
}

/*Removes a generic node of the list
 *pass the previous node
 */
void GList_removeNode(struct GenList *prev)
{
  if(!prev->next)
    {
      fprintf(stderr,"Could not remove NULL element\n");
    }
  struct GenList *temp=prev->next;
  prev->next=temp->next;
  free(temp);
  temp=NULL;
}

/*Inserts a node in the list*/
void GList_insertNode(void *item, /*the item void ptr*/
		      enum DEFTYPES tp, /*the type of the item*/
		      struct GenList *prev /*the item where to 
					     attatch the new node*/
		      )
{
  struct GenList *temp=NULL;

  //allocate the new node
  temp=malloc(sizeof(struct GenList));
  if(!temp)
    {
      fprintf(stderr,"Could not allocate memory\n");
      return;
    }

  temp->next=prev->next;
  prev->next=temp;
  temp->itemtype=tp;
  temp->itemptr=item;
}

