/*  Generic C List  Copyright (C) 2008  Giovanni Santostefano
    This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.

The hypothetical commands `show w' and `show c' should show the appropriate
parts of the General Public License.  Of course, your program's commands
might be different; for a GUI interface, you would use an "about box".

  You should also get your employer (if you work as a programmer) or school,
if any, to sign a "copyright disclaimer" for the program, if necessary.
For more information on this, and how to apply and follow the GNU GPL, see
<http://www.gnu.org/licenses/>.

  The GNU General Public License does not permit incorporating your program
into proprietary programs.  If your program is a subroutine library, you
may consider it more useful to permit linking proprietary applications with
the library.  If this is what you want to do, use the GNU Lesser General
Public License instead of this License.  But first, please read
<http://www.gnu.org/philosophy/why-not-lgpl.html>.
*/


/* Test program for the generic list.
 * Here is shown a mixed element list usage.
 * A list with mixed types element is a BAD programming style IMHO.
 * You can implement similar list interface to make a programmer able to
 * construct generic lists without mixed types elements.
 * A simple way to do that is to make a list struct that contains informations
 * about the type and a pointer to an head node and a node structure 
 * that contains only the node item pointer
 * and the next pointer.
 * In this way you also save the node type info for each node instance.
 */

#include "genlist.h"
#include "usertypes.h"

/* Fill the list */
void ListFill(struct GenList **list)
{
  int *age=NULL;
  float *height=NULL;
  struct Speed *sp=NULL;

  age=malloc(sizeof(int));
  height=malloc(sizeof(float));
  sp=malloc(sizeof(struct Speed));

  /* Test if some malloc has failed */
  if(!age || !height || !sp)
    {
      fprintf(stderr,"Some malloc has failed!!!\n");
      if(age)free(age);
      if(height)free(height);
      if(sp)free(sp);
      return;
    }

  /* value filling */
  *age=35;
  *height=1.89f;
  sp->value=101.3f;
  sp->direction=341.2f;

  /* node creation */
  *list=GList_addNode((void*)age,TYPE_INT,*list);
  *list=GList_addNode((void*)height,TYPE_FLOAT,*list);
  *list=GList_addNode((void*)sp,TYPE_SPEED,*list);
}


int main(int argc, char *argv[])
{
  struct GenList *list=NULL;
  struct GenList *temp=NULL;
  int example=0;

  ListFill(&list);

  /* Print the list */
  temp=list;
  while(temp)
    {
      /* we pass through the list.
       * For each node we test the type.
       * When we find the correct type we use the right code.
       */

      if(temp->itemtype==TYPE_INT) /* With this test we find that the 
				    * current node is of int type
				    */
	{
	  /* Here in the example we take the int item of the list
	   * and we store it in an int variable.
	   * To take the element we first make a cast of the pointer
	   * to the item to his correct type (int*) and then we use
	   * the dereference operator * to take the value pointed by
	   * itemptr
	   */
	  example=*(int*)temp->itemptr;

	  /* In this example we print directly the value */
	  printf("This element is an int\n");
	  printf("It is: %d\n",*(int*)temp->itemptr);
	}
      else if(temp->itemtype==TYPE_FLOAT)
	{
	  printf("This element is a float\n");
	  printf("It is: %f\n",*(float*)temp->itemptr);
	}
      else if(temp->itemtype==TYPE_SPEED)
	{
	  printf("This element is a struct Speed\n");
	  printf("It's value is: %f\n",((struct Speed*)temp->itemptr)->value);
	  printf("It's direction is: %f\n",
		 ((struct Speed*)temp->itemptr)->direction);
	}
      temp=temp->next;
    }

  /* Let's clear the list */
  while(list)
    {
      list=GList_removeHeadNode(list);
    }

  /* end the execution */
  return 0;
}
